# Generated by Django 3.1 on 2021-01-19 14:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0012_auto_20210119_1308'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tokenrelatedobject',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='tokenrelatedobject',
            name='token',
        ),
        migrations.RemoveField(
            model_name='usertokentakeover',
            name='token',
        ),
        migrations.RemoveField(
            model_name='usertokentakeover',
            name='user',
        ),
        migrations.AlterModelOptions(
            name='mobiledevice',
            options={'verbose_name': 'mobile device', 'verbose_name_plural': 'mobile devices'},
        ),
        migrations.AlterField(
            model_name='mobiledevice',
            name='changed_at',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at'),
        ),
        migrations.AlterField(
            model_name='mobiledevice',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at'),
        ),
        migrations.DeleteModel(
            name='Token',
        ),
        migrations.DeleteModel(
            name='TokenRelatedObject',
        ),
        migrations.DeleteModel(
            name='UserTokenTakeover',
        ),
        migrations.AlterField(
            model_name='authorizationtoken',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at'),
        ),
        migrations.AlterField(
            model_name='authorizationtoken',
            name='changed_at',
            field=models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at'),
        ),
    ]
