# Generated by Django 3.1 on 2021-03-08 08:18

from django.db import migrations, models
from django.conf import settings
import auth_token.enums
import auth_token.models
import django.db.models.deletion
import enumfields.fields


class Migration(migrations.Migration):

    dependencies = [
        ('auth_token', '0013_auto_20210119_1547'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='authorizationrequestgenericmanytomanyrelation',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='authorizationrequestgenericmanytomanyrelation',
            name='authorization_request',
        ),
        migrations.RemoveField(
            model_name='authorizationrequestgenericmanytomanyrelation',
            name='object_ct',
        ),
        migrations.AlterUniqueTogether(
            name='onetimepasswordgenericmanytomanyrelation',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='onetimepasswordgenericmanytomanyrelation',
            name='object_ct',
        ),
        migrations.RemoveField(
            model_name='onetimepasswordgenericmanytomanyrelation',
            name='one_time_password',
        ),
        migrations.AlterUniqueTogether(
            name='authorizationtokengenericmanytomanyrelation',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='authorizationtokengenericmanytomanyrelation',
            name='object_ct',
        ),
        migrations.RemoveField(
            model_name='authorizationtokengenericmanytomanyrelation',
            name='authorization_token',
        ),
        migrations.AddField(
            model_name='authorizationtoken',
            name='id',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.DeleteModel(
            name='AuthorizationRequest',
        ),
        migrations.DeleteModel(
            name='AuthorizationRequestGenericManyToManyRelation',
        ),
        migrations.DeleteModel(
            name='OneTimePassword',
        ),
        migrations.DeleteModel(
            name='OneTimePasswordGenericManyToManyRelation',
        ),
        migrations.DeleteModel(
            name='AuthorizationTokenGenericManyToManyRelation',
        ),
        migrations.DeleteModel(
            name='UserAuthorizationTokenTakeover',
        ),
        migrations.DeleteModel(
            name='AuthorizationToken'
        )
    ]
